
clear variables;close all;clc

rng('default');                               % for reproducible random number generation

% Read ASVAB data
a = readmatrix('data\asvab.csv','NumHeaderLines',1); % raw ASVAB data (12686 x 14 matrix)
asvabScores = a(:,2:11);                     % extract score columns (R0615000, R0615100, R0615200, R0615300, R0615400, R0615500, R0615600, R0615700, R0615800, R0615900)
asvabGender = a(:,1);                        % read gender codes (R0214800) (1 = male, 2 = female)
mask = any(asvabScores<0,2);                 % identify rows with any negative (missing) values
asvabScores(mask,:) = [];                    % remove those rows from scores (11914 x 10 matrix). Select the 7th ASVAB test
asvabGender(mask) = [];                      % remove corresponding gender entries (11914 x 1 vector)

% Read DBQ data
useDbqCsv = false;                               % set false to force rebuild
csvFile   = 'data\dbq.csv';

if useDbqCsv && exist(csvFile,'file')            % CSV ---
    b = readmatrix(csvFile,'NumHeaderLines',1);  %#ok<UNRCH> % gender + Aggeessive Violations Scale scores
else                                             % MAT ---
    load('data\deq1.mat');                       % load matrix deq1
    deq1(deq1<0) = NaN;                          % negatives → NaN
    v = ~any(isnan(deq1(:,[40 52:88 107])),2);   % valid rows
    b = [deq1(v,6) 6*deq1(v,226)];               % extract gender + Aggressive Violations Scale 
    % Multiplied by 6, since the Aggressive Violations Scale consists of 6 items. Now the score consists of whole numbers
    b = sortrows(b,2);                           % sort rows by the Aggressive Violations Scale score (2nd column) (prevents identification)
    hdr = {'Gender (1 = female, 2 = male)', 'Aggressive Violations Scale score'};
    fid = fopen(csvFile,'w'); fprintf(fid,'%s\n',strjoin(hdr,',')); fclose(fid);
    writematrix(b,csvFile,'WriteMode','append'); % cache as CSV
end

dbqGender = b(:,1);                             % 9077 × 1
dbqScale = b(:,2);                              % 9077 × 1

%%
reps = 10^6;
N2 = 2:50;
[pIT, pWT, pITm, pWTm,CorrectSign] = deal(NaN(4, length(N2), reps));
TPperformance = NaN(4, 2);

for m = 1:4
    if m <= 2  % ASVAB tests (1=Male, 2=Female)
        score = asvabScores;
        males = find(asvabGender == 1);
        females = find(asvabGender == 2);
    else  % DBQ aggressive violations (2=Male, 1=Female)
        score = dbqScale; 
        males = find(dbqGender == 2);
        females = find(dbqGender == 1);
    end
    males_higher = sign( mean(score(males)) - mean(score(females) ));
    % Display descriptive statistics for the first subfigure of each pair
    if mod(m, 2) == 1
        %% Table 1. Descriptive statistics of the two datasets, distinguishing between males and females
        statNames = {'n'; 'Minimum'; 'Maximum'; 'Mean'; 'SD'; 'Skewness'; 'Kurtosis'};
        statsFuncs = {@length; @min; @max; @mean; @std; @skewness; @kurtosis};
        maleStats = cellfun(@(f) f(score(males)), statsFuncs);
        femaleStats = cellfun(@(f) f(score(females)), statsFuncs);

        rowsToRound = 4:7;
        maleStats(rowsToRound) = round(maleStats(rowsToRound) * 100) / 100;
        femaleStats(rowsToRound) = round(femaleStats(rowsToRound) * 100) / 100;
        T = table(maleStats, femaleStats, 'RowNames', statNames, 'VariableNames', {'Males', 'Females'});
        datasetMap = {'ASVAB - Auto & Shop information', 'ASVAB - Auto & Shop information', ...
            'DBQ - Aggressive violations', 'DBQ - Aggressive violations'};
        datasetName = datasetMap{m};

        disp(' ');
        disp(['Table 1. Descriptive statistics: ', datasetName]);
        disp(T); % MATLAB's table display will show the stored precision
        disp(newline)
        disp('Corresponding distribution')
        disp([ (0:36)', histcounts(score(males), 0:37)', histcounts(score(females), 0:37)' ]) % histogram of distribution of males and females

    end

    for j = 1:length(N2)
        n1 = 50;
        n2 = N2(j);
        if m == 1 || m == 3
            idx_males = males(randi(length(males), n1, reps));
            idx_females = females(randi(length(females), n2, reps));
        elseif m == 2 || m == 4
            idx_males = males(randi(length(males), n2, reps));
            idx_females = females(randi(length(females), n1, reps));
        end

        % Perform t-test and Welch's test between genders
        X1 = score(idx_males);
        X2 = score(idx_females);

        [pIT(m, j, :), pWT(m, j, :)] = ttest2_and_welch(X1, X2);

        CorrectSign(m, j, :) = (mean(X1) > mean(X2) & males_higher == 1) | (mean(X2) > mean(X1) & males_higher == -1);

        % Compare within the same gender groups
        idx_males2 = males(randi(length(males), n2, reps));
        idx_females2 = females(randi(length(females), n2, reps));
        if m == 1 || m == 3
            X1 = score(idx_males);
            X2 = score(idx_males2);
        elseif m == 2 || m == 4
            X1 = score(idx_females);
            X2 = score(idx_females2);
        end
        [pITm(m, j, :),pWTm(m, j, :)] =  ttest2_and_welch(X1, X2);
    end
    TPperformance(m, :) = 100 * [mean(mean(squeeze(pIT(m, :, :))' < 0.05 & squeeze(CorrectSign(m, :, :))' == 1)) mean(mean(squeeze(pWT(m, :, :))' < 0.05 & squeeze(CorrectSign(m, :, :))' == 1))];
end

% Figure 3. Left figures: Power when comparing a fixed sample of 50 males against a variable sample of females (shown on the x-axis). Right figures: Power when comparing a fixed sample of 50 females against a variable sample of males (shown on the x-axis).
figure
set(gcf,'renderer','painters')
for m = 1:4
    subplot(2, 2, m)
    plot(N2, mean(squeeze(pIT(m, :, :))' < 0.05 & squeeze(CorrectSign(m, :, :))' == 1), 'k-o', 'LineWidth', 3, 'markerfacecolor', 'k'); hold on
    plot(N2, mean(squeeze(pWT(m, :, :))' < 0.05 & squeeze(CorrectSign(m, :, :))' == 1), 'm-o', 'LineWidth', 3, 'markerfacecolor', 'm')
    disp('Overall true positive rate of IT and WT, i.e., averaged across all sample sizes and repetitions')
    disp(TPperformance(m, :))
    if m == 1
        title('ASVAB - Auto-shop information, \it{n_m_a_l_e_s}\rm = 50')
    elseif m == 2
        title('ASVAB - Auto-shop information, \it{n_f_e_m_a_l_e_s}\rm = 50')
    elseif m == 3
        title('DBQ - Aggressive violations, \it{n_m_a_l_e_s}\rm = 50')
    elseif m == 4
        title('DBQ - Aggressive violations, \it{n_f_e_m_a_l_e_s}\rm = 50')
    end
    legend('\it{t}\rm-test','Welch''s test','Location','best')
    grid on
    set(gca, 'xlim', [1 N2(end)])
    set(gca, 'xtick', [2 10 20 30 40 50])
    ylabel('Proportion \it{p}\rm < 0.05')
    if m == 1 || m == 3
        xlabel('\it{n_f_e_m_a_l_e_s}\rm');
    else
        xlabel('\it{n_m_a_l_e_s}\rm');
    end
    if m == 1
        set(gca, 'Position', [0.04 0.58 0.43 0.37])
    elseif m == 2
        set(gca, 'Position', [0.56 0.58 0.43 0.37])
    elseif m == 3
        set(gca, 'Position', [0.04 0.075 0.43 0.37])
    elseif m == 4
        set(gca, 'Position', [0.56 0.075 0.43 0.37])
    end
end
h = findobj('FontName', 'Helvetica');
set(h, 'FontSize', 18, 'FontName', 'Arial');

%% Figure 4. Left figures: False positive rate when comparing a fixed sample of 50 males against a second sample of males with size n2 (shown on the x-axis). Right figures: Comparison of a fixed sample of 50 females against a second sample of females with size n2 (shown on the x-axis).
figure
FPperformance = zeros(4, 2);
for m = 1:4
    subplot(2, 2, m)
    plot(N2, mean(squeeze(pITm(m, :, :))' < 0.05), 'k-o', 'LineWidth', 3, 'markerfacecolor', 'k'); hold on
    plot(N2, mean(squeeze(pWTm(m, :, :))' < 0.05), 'm-o', 'LineWidth', 3, 'markerfacecolor', 'm')
    disp('False positive rate')
    FPperformance(m, :) = 100 * [mean(mean(squeeze(pITm(m, :, :))' < 0.05)) mean(mean(squeeze(pWTm(m, :, :))' < 0.05))];
    disp(FPperformance(m, :))
    if m == 1
        title('ASVAB - Auto-shop information, \it{n_1_,_m_a_l_e_s}\rm \bf{= 50}')
    elseif m == 2
        title('ASVAB - Auto-shop information, \it{n_1_,_f_e_m_a_l_e_s}\rm \bf{= 50}')
    elseif m == 3
        title('DBQ - Aggressive violations, \it{n_1_,_m_a_l_e_s}\rm \bf{= 50}')
    elseif m == 4
        title('DBQ - Aggressive violations, \it{n_1_,_f_e_m_a_l_e_s}\rm \bf{= 50}')
    end
    legend('\it{t}\rm-test','Welch''s test','Location','best')
    grid on
    set(gca, 'xlim', [1 N2(end)])
    set(gca, 'xtick', [2 10 20 30 40 50])
    ylabel('Proportion \it{p}\rm < 0.05')
    if m == 2 || m == 4
        xlabel('\it{n_2_,_f_e_m_a_l_e_s}\rm');
    else
        xlabel('\it{n_2_,_m_a_l_e_s}\rm');
    end
    if m == 1
        set(gca, 'Position', [0.05 0.58 0.43 0.37])
    elseif m == 2
        set(gca, 'Position', [0.56 0.58 0.43 0.37])
    elseif m == 3
        set(gca, 'Position', [0.05 0.075 0.43 0.37])
    elseif m == 4
        set(gca, 'Position', [0.56 0.075 0.43 0.37])
    end
end
h = findobj('FontName', 'Helvetica');
set(h, 'FontSize', 18, 'FontName', 'Arial');

%% Display ASVAB descriptive statistics for males and females
disp('ASVAB: Means and SDs for males and females')
disp(round([mean(asvabScores(asvabGender==1, :))' mean(asvabScores(asvabGender==2, :))' ...
    std(asvabScores(asvabGender==1, :))' std(asvabScores(asvabGender==2, :))' ...
    std(asvabScores(asvabGender==1, :)) ./ std(asvabScores(asvabGender==2, :))'], 2))

mdr = mean(asvabScores(asvabGender==1, :))' ./ mean(asvabScores(asvabGender==2, :))';
sdr = std(asvabScores(asvabGender==1, :))' ./ std(asvabScores(asvabGender==2, :))';

%% Figure B1. Scatter plot of the standard deviation (SD) ratio (male SD / female SD) against the mean ratio (male mean / female mean) for the 10 subtests of the Armed Services Vocational Aptitude Battery (ASVAB).
figure
set(gcf,'renderer','painters')
scatter(mdr, sdr, 400, 'markerfacecolor', 'y', 'MarkerEdgeColor', 'k', 'LineWidth', 3, 'MarkerFaceAlpha', 0.5)
for i = 1:10
    text(mdr(i), sdr(i)+0.026, num2str(i),'horizontalalignment','center')
end
ax=gca;
ax.GridLineWidth = 1.5; % Set grid line width

set(gca,'xlim',[0.8 1.55],'ylim',[0.8 1.55])
grid on; box on
xlabel('Mean ratio (mean male / mean female)')
ylabel('SD ratio (SD male / SD female)')
h = findobj('FontName', 'Helvetica');
set(h, 'FontSize', 24, 'FontName', 'Arial');
set(gca, 'Position', [0.05 0.08 0.45 0.8])
disp('ASVAB - Mean true positive rate for IT and WT')
disp([mean(TPperformance(1:2, 1)) mean(TPperformance(1:2, 2))])
disp('DBQ - Mean true positive rate for IT and WT')
disp([mean(TPperformance(3:4, 1)) mean(TPperformance(3:4, 2))])
