clear; close all; clc;
tic
%% Simulation Setup

% conditionParams columns:
%   1. meanDiff       (Mean difference between Group 1 and Group 2)
%   2. sdGroup1       (Standard deviation of Group 1)
%   3. sdGroup2       (Standard deviation of Group 2)
%   4. distributionID (0: normal, 1: lognormal)

conditionParams = [ ...
    0   2   1   0;   % Condition A: Type I error,   sdGroup1 = 2, normal
    0   1   2   0;   % Condition B: Type I error,   sdGroup2 = 2, normal
    0   1   1   0;   % Condition C: Type I error,   sdGroup1 = sdGroup2 = 1, normal
    0   1   1   1;   % Condition D: Type I error,   sdGroup1 = sdGroup2 = 1, lognormal
    1   2   1   0;   % Condition E: Power,          sdGroup1 = 2, normal
    1   1   2   0;   % Condition F: Power,          sdGroup2 = 2, normal
    1   1   1   0;   % Condition G: Power,          sdGroup1 = sdGroup2 = 1, normal
    1   1   1   1];  % Condition H: Power,          sdGroup1 = sdGroup2 = 1, lognormal

% Titles for Figure 1 (t-test and Welch)
fig1Titles = { ...
    'FPR, SD_1/SD_2 = 2, normal', ...
    'FPR, SD_2/SD_1 = 2, normal', ...
    'FPR, SD_1 = SD_2 = 1, normal', ...
    'FPR, SD_1 = SD_2 = 1, lognormal', ...
    'TPR, SD_1/SD_2 = 2, normal', ...
    'TPR, SD_2/SD_1 = 2, normal', ...
    'TPR, SD_1 = SD_2 = 1, normal', ...
    'TPR, SD_1 = SD_2 = 1, lognormal'};

% Titles for Figure 2 (Kolmogorov–Smirnov and Anderson–Darling)
fig2Titles = { ...
    'TPR, SD_1/SD_2 = 2, normal', ...
    'TPR, SD_2/SD_1 = 2, normal', ...
    'FPR, SD_1 = SD_2 = 1, normal', ...
    'FPR, SD_1 = SD_2 = 1, lognormal', ...
    'TPR, SD_1/SD_2 = 2, normal', ...
    'TPR, SD_2/SD_1 = 2, normal', ...
    'TPR, SD_1 = SD_2 = 1, normal', ...
    'TPR, SD_1 = SD_2 = 1, lognormal'};

% Lognormal parameters
lognormMean = 0.8;     % mean of lognormal distribution
lognormVar  = 1;       % variance of lognormal distribution
mu    = log((lognormMean^2) / sqrt(lognormVar + lognormMean^2));
sigma = sqrt(log(lognormVar/(lognormMean^2) + 1));

% Sample size for Group 1 (fixed), and range for Group 2
nGroup1      = 50;
%nGroup1      = 1000; % used for creating Figure A1
nGroup2Range = 2:50;

% Number of replications for normal-based tests vs. distribution tests
numReps        = 1e6;     % For t-test and Welch
numRepsSmaller = 25000;   % For Kolmogorov–Smirnov and Anderson–Darling

%% Preallocate storage for p-values
pValsTtest      = cell(size(conditionParams,1),1);
pValsWelch      = cell(size(conditionParams,1),1);
pValsKolmog     = cell(size(conditionParams,1),1);
pValsAD         = cell(size(conditionParams,1),1);

%% Run Simulation

for condIdx = 1:size(conditionParams, 1)

    % Extract condition-specific parameters
    meanDiff       = conditionParams(condIdx, 1);
    sdGroup1       = conditionParams(condIdx, 2);
    sdGroup2       = conditionParams(condIdx, 3);
    isLognormal    = (conditionParams(condIdx, 4) == 1);

    % Pre-generate random numbers for Group 1
    randGroup1 = gpuArray.randn(nGroup1, numReps);

    % Preallocate matrices for p-values (rows: nGroup2, cols: replications)
    pValTtestMat  = NaN(length(nGroup2Range), numReps);
    pValWelchMat  = NaN(length(nGroup2Range), numReps);
    pValKolmogMat = NaN(length(nGroup2Range), numRepsSmaller);
    pValADMat     = NaN(length(nGroup2Range), numRepsSmaller);

    for n2Idx = 1:length(nGroup2Range)
        currN2 = nGroup2Range(n2Idx);

        % Generate random numbers for Group 2 (current sample size)
        randGroup2 = gpuArray.randn(currN2, numReps);

        % Create data based on the specified distribution
        if ~isLognormal
            % Normal distribution
            dataGroup1 = randGroup1 * sdGroup1 + meanDiff;
            dataGroup2 = randGroup2 * sdGroup2;
        else
            % Lognormal distribution
            dataGroup1 = exp(randGroup1 * sigma + mu) + meanDiff;
            dataGroup2 = exp(randGroup2 * sigma + mu);
        end

        % Run t-test & Welch’s test
        [pttest2, PWelch] = ttest2_and_welch(dataGroup1, dataGroup2);

        % Store in current condition’s matrix
        pValTtestMat(n2Idx, :) = pttest2;
        pValWelchMat(n2Idx, :) = PWelch;

        % Run Kolmogorov–Smirnov & Anderson–Darling tests
        % These functions operate on a smaller subset (numRepsSmaller)
        pKolmogorov = kstest2_short(dataGroup1(:,1:numRepsSmaller), dataGroup2(:,1:numRepsSmaller));
        pAD = adtest2_ties_fast(gather(dataGroup1(:,1:numRepsSmaller)), gather(dataGroup2(:,1:numRepsSmaller)));
        % Store in current condition’s matrix
        pValKolmogMat(n2Idx, :) = pKolmogorov;
        pValADMat(n2Idx, :)     = pAD;

        % Status update in the Command Window
        disp(['Condition ' num2str(condIdx) ', n2 = ' num2str(currN2) ' done.']);
    end

    % Store results in cell arrays
    pValsTtest{condIdx}  = pValTtestMat;
    pValsWelch{condIdx}  = pValWelchMat;
    pValsKolmog{condIdx} = pValKolmogMat;
    pValsAD{condIdx}     = pValADMat;
end

%load pValues_Kolmog_AD.mat % load p-values of previously conducted simulations
toc
%% Figure 2. Simulation results for the independent-samples t-test (IT) and Welch’s test (WT)
fig1 = figure('Name','Figure1 - ttest & welch','Renderer','painters'); %#ok<*FGREN>
tiledlayout(2,4);
fig1.WindowState = 'maximized';
tiledlayout(2,4, 'Padding','none', 'TileSpacing','loose');

for condIdx = 1:size(conditionParams,1)

    meanDiff = conditionParams(condIdx,1);

    nexttile;
    hold on; grid on;

    % Proportion of p-values < 0.05
    propTtest = mean(pValsTtest{condIdx} < 0.05, 2);
    propWelch = mean(pValsWelch{condIdx} < 0.05, 2);

    plot(nGroup2Range, propTtest, 'k-o','LineWidth',2,'MarkerFaceColor','k');
    plot(nGroup2Range, propWelch, 'm-o','LineWidth',2,'MarkerFaceColor','m');

    if meanDiff == 0
        legend('\it{t}\rm-test', 'Welch''s test','Location','northeast');
        ylim([0 0.4]);
    else
        legend('\it{t}\rm-test', 'Welch''s test','Location','southeast');
        ylim([0 1]);
    end

    xlim([min(nGroup2Range) max(nGroup2Range)]);
    set(gca,'XTick',[2 10 20 30 40 50]);
    xlabel('\it{n}_2');
    ylabel('Proportion \it{p}\rm < 0.05');
    title(fig1Titles{condIdx});

    % Label subplots (A), (B), etc.
    text(0.04, 0.94, ['(' char(64 + condIdx) ')'], ...
        'Units','normalized', 'FontSize',14, ...
        'Color','b', 'FontWeight','bold');
end

% Global formatting
h = findall(gcf,'-property','FontName');
set(h,'FontName','Arial','FontSize',18);

%% Figure 5. Simulation results for the Kolmogorov–Smirnov test (KST) and the Anderson–Darling test (ADT)
fig2 = figure('Name','Figure2','Renderer','painters');
fig2.WindowState = 'maximized';
tiledlayout(2,4, 'Padding','none', 'TileSpacing','loose');

for condIdx = 1:size(conditionParams,1)

    meanDiff = conditionParams(condIdx,1);

    nexttile;
    hold on; grid on;

    propKolmogorov = mean(pValsKolmog{condIdx} < 0.05, 2);
    propAD         = mean(pValsAD{condIdx}     < 0.05, 2);

    plot(nGroup2Range, propKolmogorov, 'k-o','LineWidth',2,'MarkerFaceColor','k');
    plot(nGroup2Range, propAD,         'r-o','LineWidth',2,'MarkerFaceColor','r');

    % Adjust legend/ylim depending on condition
    if ismember(condIdx,[3 4])
        legend('Kolmogorov–Smirnov','Anderson–Darling','Location','northeast');
        ylim([0 0.4]);
    elseif ismember(condIdx,[1 2])
        legend('Kolmogorov–Smirnov','Anderson–Darling','Location','northeast');
        ylim([0 1]);
    else
        legend('Kolmogorov–Smirnov','Anderson–Darling','Location','southeast');
        ylim([0 1]);
    end

    xlim([min(nGroup2Range) max(nGroup2Range)]);
    set(gca,'XTick',[2 10 20 30 40 50]);
    xlabel('\it{n}_2');
    ylabel('Proportion \it{p}\rm < 0.05');
    title(fig2Titles{condIdx});

    if condIdx == 8
        text(0.14, 0.91, ['(' char(64 + condIdx) ')'], ...
            'Units','normalized', 'FontSize',14, ...
            'Color','b', 'FontWeight','bold');
    else
        text(0.04, 0.94, ['(' char(64 + condIdx) ')'], ...
            'Units','normalized', 'FontSize',14, ...
            'Color','b', 'FontWeight','bold');
    end
end

% Global formatting
h = findall(gcf,'-property','FontName');
set(h,'FontName','Arial','FontSize',18);

%% Figure 1. Skewed distribution used for Conditions D and H
% Compute population skewness of the lognormal distribution
popSkewness = (exp(sigma^2) + 2) * sqrt(exp(sigma^2) - 1);
disp(['Population skewness of the lognormal: ' num2str(popSkewness)]);

vecX    = -10:0.001:20;
pdfVals = lognpdf(vecX, mu, sigma);

figure('Name','Figure - Lognormal PDF','Renderer','painters');
plot(vecX, pdfVals, 'k','LineWidth',3);
grid on;
xlabel('Value');
ylabel('Probability density');
xlim([-0.3 5]);

h = findall(gcf,'-property','FontName');
set(h,'FontName','Arial','FontSize',18);
set(gca,'LooseInset',[0.01,0.01,0.01,0.01]);
