function pVal = adtest2_ties_fast(x,y,MCReps)

% Two-sample Anderson–Darling permutation test with tie correction.
% Corrects for ties, based on

% Pettitt, A. N. (1976). A two-sample Anderson-Darling rank statistic. Biometrika, 63(1), 161–168. https://doi.org/10.1093/biomet/63.1.161
% Scholz, F. W., & Stephens, M. A. (1987). K-Sample Anderson-Darling Tests. Journal of the American Statistical Association, 82(399), 918–924. https://doi.org/10.1080/01621459.1987.10478517

% INPUTS:
%   x, y: Data samples (vectors or columns of matrices)
%   MCReps: Number of Monte Carlo repetitions (default: 1000)
% OUTPUT:
%   pVal: Permutation p-value(s)

% -------------------------------------------------------------------------
if nargin<3 || isempty(MCReps), MCReps = 1e3; end

if isvector(x), x = x(:); end
if isvector(y), y = y(:); end
[~,M] = size(x);
if size(y,2) ~= M
    error('x and y must have the same number of columns');
end
pVal = NaN(1,M);

for c = 1:M
    xi = x(:,c);   xi = xi(~isnan(xi));
    yi = y(:,c);   yi = yi(~isnan(yi));
    Nx = numel(xi);   Ny = numel(yi);   N = Nx+Ny;
    if Nx==0 || Ny==0 || N<2,  continue,  end          % leave NaN

    % ---------- sort pooled data & original labels ---------------------
    [sorted_vals,sortIdx] = sort([xi;yi]);
    sorted_labels = [true(Nx,1); false(Ny,1)];
    sorted_labels = sorted_labels(sortIdx);

    % ---------- tie structure (depends only on sorted_vals) ------------
    [~,~,ic]  = unique(sorted_vals,'stable');
    t_j       = accumarray(ic,1);                      % group sizes
    i_j       = accumarray(ic,(1:N)',[],@max);         % end indices
    keep      = i_j < N;                               % last group never used
    i_j       = i_j(keep);
    t_j       = t_j(keep);
    denomTerm = (i_j./N).*(1 - i_j./N);
    tj_divDen = t_j ./ denomTerm;                      % t_j / denom

    % ---------- observed statistic ------------------------------------
    cumsum_x  = cumsum(sorted_labels);
    M_ij_obs  = cumsum_x(i_j);
    obsAD     = (N/(Nx*Ny)) * ...
                sum( tj_divDen .* (M_ij_obs - i_j*Nx/N).^2 );

    % ---------- all permutation label matrices at once ----------------
    % sort a random matrix column-wise; the rows 1:Nx become the "x" sample
    [~,ord] = sort(rand(N,MCReps),1);      % N×MCReps indices
    P       = ord<=Nx;                     % logical labels
    P_sorted = P(sortIdx,:);               % match sorted order

    % cumulative counts of sample-1 across rows, for every permutation
    C          = cumsum(P_sorted,1);       % N×MCReps
    M_ij_perm  = C(i_j,:);                 % (#groups)×MCReps

    A2_sum_perm = sum( tj_divDen .* ...
                      (M_ij_perm - i_j*(Nx/N)).^2 , 1 );
    permAD      = (N/(Nx*Ny)) * A2_sum_perm;

    % ---------- p-value ------------------------------------------------
    pVal(c) = mean(permAD >= obsAD);
end
end
