function [pttest2, pwelch] = ttest2_and_welch(X1, X2)

%   Inputs:
%       X1, X2 : Matrices of size (n x m), where:
%                - n is the number of observations in each group,
%                - m is the number of replications (e.g., columns of data)
%   Outputs:
%       pttest2 : 1 x m array of two-sided p-values (equal-variance t-test)
%       pwelch  : 1 x m array of two-sided p-values (Welch’s t-test).

    % 1) Basic dimensions
    n1 = size(X1,1);  % # of rows in X1
    n2 = size(X2,1);  % # of rows in X2

    % 2) Column-wise means & variances
    m1 = mean(X1, 1);    % 1 x m
    m2 = mean(X2, 1);    % 1 x m
    v1 = var(X1, 0, 1);  % 1 x m
    v2 = var(X2, 0, 1);  % 1 x m

    diffVal = m1 - m2;   % 1 x m difference in means

    % ---------------------------------------------------------------------
    % Independent-samples t-test (equal variances assumed)
    % ---------------------------------------------------------------------
    dfe = n1 + n2 - 2;    % degrees of freedom for pooled-variance t-test
    sPooled = sqrt( ((n1 - 1).*v1 + (n2 - 1).*v2) ./ dfe ); % Pooled variance estimate
    se = sPooled .* sqrt(1/n1 + 1/n2); % Standard error of mean difference
    tVal = diffVal ./ se; % t-statistic for equal variances
    pttest2 = 2 * tcdf(-abs(tVal), dfe); % Two-sided p-values (using t-distribution)

    % ---------------------------------------------------------------------
    % Welch’s t-test (unequal variances)
    % ---------------------------------------------------------------------
    var1bar = v1 / n1;  
    var2bar = v2 / n2;  
    dfW = (var1bar + var2bar).^2 ./ (var1bar.^2 / (n1 - 1) + var2bar.^2 / (n2 - 1)); % Welch–Satterthwaite effective degrees of freedom
    seWelch = sqrt(var1bar + var2bar); % Standard error for Welch’s
    tValWelch = diffVal ./ seWelch; % Welch’s t-statistic
    pwelch = 2 * tcdf(-abs(tValWelch), dfW); % Two-sided p-values (using Welch’s df)

end
